pub type HashMLDSA65_with_SHA512 = HashMLDSA<SHA512, 64, MLDSA65_PK_LEN, MLDSA65_SK_LEN, MLDSA65_SIG_LEN, MLDSA65PublicKey, MLDSA65PrivateKey, MLDSA65_TAU, MLDSA65_LAMBDA, MLDSA65_GAMMA1, MLDSA65_GAMMA2, MLDSA65_k, MLDSA65_l, MLDSA65_ETA, MLDSA65_BETA, MLDSA65_OMEGA, MLDSA65_C_TILDE, MLDSA65_POLY_Z_PACKED_LEN, MLDSA65_POLY_W1_PACKED_LEN, MLDSA65_LAMBDA_over_4, MLDSA65_GAMMA1_MINUS_BETA, MLDSA65_GAMMA2_MINUS_BETA, MLDSA65_GAMMA1_MASK_LEN>;Expand description
The HashML-DSA-65_with_SHA512 signature algorithm.
Aliased Type
pub struct HashMLDSA65_with_SHA512 { /* private fields */ }Trait Implementations
Sourceimpl Algorithm for HashMLDSA65_with_SHA512
impl Algorithm for HashMLDSA65_with_SHA512
Sourceconst ALG_NAME: &'static str = HASH_ML_DSA_65_WITH_SHA512_NAME
const ALG_NAME: &'static str = HASH_ML_DSA_65_WITH_SHA512_NAME
String name for the algorithm, used consistently across the library.
Sourceconst MAX_SECURITY_STRENGTH: SecurityStrength = SecurityStrength::_192bit
const MAX_SECURITY_STRENGTH: SecurityStrength = SecurityStrength::_192bit
Maximum security strength supported by the algorithm.
In other words, this algorithm can produce outputs up to this security strength,
but may produce outputs with lower security strength, for example, if asked to truncate.
Sourceimpl AlgorithmOID for HashMLDSA65_with_SHA512
Assigned by NIST in the Computer Security Objects Register: id-hash-ml-dsa-65-with-sha512 { sigAlgs 33 }
impl AlgorithmOID for HashMLDSA65_with_SHA512
Assigned by NIST in the Computer Security Objects Register: id-hash-ml-dsa-65-with-sha512 { sigAlgs 33 }