pub type HashMLDSA87_with_SHA256 = HashMLDSA<SHA256, 32, MLDSA87_PK_LEN, MLDSA87_SK_LEN, MLDSA87_SIG_LEN, MLDSA87PublicKey, MLDSA87PrivateKey, MLDSA87_TAU, MLDSA87_LAMBDA, MLDSA87_GAMMA1, MLDSA87_GAMMA2, MLDSA87_k, MLDSA87_l, MLDSA87_ETA, MLDSA87_BETA, MLDSA87_OMEGA, MLDSA87_C_TILDE, MLDSA87_POLY_Z_PACKED_LEN, MLDSA87_POLY_W1_PACKED_LEN, MLDSA87_LAMBDA_over_4, MLDSA87_GAMMA1_MINUS_BETA, MLDSA87_GAMMA2_MINUS_BETA, MLDSA87_GAMMA1_MASK_LEN>;Expand description
The HashML-DSA-87_with_SHA256 signature algorithm.
Aliased Type
pub struct HashMLDSA87_with_SHA256 { /* private fields */ }Trait Implementations
Sourceimpl Algorithm for HashMLDSA87_with_SHA256
impl Algorithm for HashMLDSA87_with_SHA256
Sourceconst ALG_NAME: &'static str = HASH_ML_DSA_87_with_SHA256_NAME
const ALG_NAME: &'static str = HASH_ML_DSA_87_with_SHA256_NAME
String name for the algorithm, used consistently across the library.
Sourceconst MAX_SECURITY_STRENGTH: SecurityStrength = SecurityStrength::_128bit
const MAX_SECURITY_STRENGTH: SecurityStrength = SecurityStrength::_128bit
Maximum security strength supported by the algorithm.
In other words, this algorithm can produce outputs up to this security strength,
but may produce outputs with lower security strength, for example, if asked to truncate.